#include "AINetworkArcSelection.h"
#include "AINetworkArc.h"
#include "AINetworkActionHandler.h"
#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(AINetworkArcSelection,MapSelection);

AINetworkArcSelection::AINetworkArcSelection(MapSelectionManager* manager, AINetworkArc* content):
        MapSelection(manager,content) {
}

AINetworkArcSelection::~AINetworkArcSelection() {
}

bool AINetworkArcSelection::Hit(Point3D p) {
        return GetContent()->Hit(GetEditor(),TDRectangle(p,p));
}

wxRect AINetworkArcSelection::GetDeviceBounds() const {
        wxRect r=GetContent()->GetDeviceBounds(GetEditor());
        return r;
}

void AINetworkArcSelection::ProvideActionMenuForClass(wxMenu& popupMenu, wxClassInfo* targetClass) {
        MapSelection::ProvideActionMenuForClass(popupMenu,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkArcSelection)))
                return;
        wxMenu* vertexMenu=new wxMenu();
        vertexMenu->SetEventHandler(popupMenu.GetEventHandler());
        vertexMenu->SetClientData(popupMenu.GetClientData());
        wxMenuItem *item;
        
        item=vertexMenu->Append(wxID_ANY,wxT("Flip unidirectional arcs"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnFlipUnidirectionalEdges),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        item=vertexMenu->Append(wxID_ANY,wxT("Split arcs at center"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnSplitAtCenter),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        popupMenu.Append(wxID_HIGHEST,wxT("AI Arcs"),vertexMenu);
}

void AINetworkArcSelection::DeltaDrag(Point3D delta) {
        // Drag our end nodes
        MapProjection* proj=GetEditor()->GetProjection();
        
        Point3D sourcePos,targetPos;
        sourcePos=GetContent()->GetSource()->GetPosition();
        targetPos=GetContent()->GetTarget()->GetPosition();
        
        Point3D localSourcePos,localTargetPos;
        localSourcePos=proj->ConvertToLocal(sourcePos);
        localTargetPos=proj->ConvertToLocal(targetPos);
        
        localSourcePos+=delta;
        localTargetPos+=delta;
        
        sourcePos=proj->ConvertFromLocal(localSourcePos);
        targetPos=proj->ConvertFromLocal(localTargetPos);
        
        GetContent()->GetSource()->SetPosition(sourcePos);
        GetContent()->GetTarget()->SetPosition(targetPos);
}

wxString AINetworkArcSelection::EndDrag() {
        return wxT("Moving AI network arc");
}

void AINetworkArcSelection::Paint(wxDC& dc) {
        GetContent()->PaintArc(dc,GetEditor(),Selection_Marking_Colour);
}

void AINetworkArcSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkArcSelection)))
                return;
        
        propgrid->Append(wxStringProperty(wxT("Name"), wxT("AINetworkArcSelection_Name")));
        propgrid->Append(wxBoolProperty(wxT("Is Bidirectional"), wxT("AINetworkArcSelection_IsBidirectional")));
        propgrid->Append(wxBoolProperty(wxT("Is Pushback route"), wxT("AINetworkArcSelection_IsPushBackRoute")));
}

void AINetworkArcSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkArcSelection)))
                return;

        propgrid->SetPropertyValue(wxT("AINetworkArcSelection_Name"),GetContent()->GetName());
        propgrid->SetPropertyValue(wxT("AINetworkArcSelection_IsBidirectional"),GetContent()->IsBidirectional());
        propgrid->SetPropertyValue(wxT("AINetworkArcSelection_IsPushBackRoute"),GetContent()->IsPushBackRoute());
}

void AINetworkArcSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        MapSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkArcSelection)))
                return;
        AINetworkArc* firstContent=wxDynamicCast(firstSelection,AINetworkArcSelection)->GetContent();
        AINetworkArc* myContent=GetContent();

        if (firstContent->GetName().Cmp(myContent->GetName())!=0)
                propgrid->SetPropertyUnspecified(wxT("AINetworkArcSelection_Name"));
        if (firstContent->IsBidirectional()!=myContent->IsBidirectional())
                propgrid->SetPropertyUnspecified(wxT("AINetworkArcSelection_IsBidirectional"));
        if (firstContent->IsPushBackRoute()!=myContent->IsPushBackRoute())
                propgrid->SetPropertyUnspecified(wxT("AINetworkArcSelection_IsPushBackRoute"));
}

void AINetworkArcSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("AINetworkArcSelection_IsBidirectional"))==0) {
                GetContent()->SetIsBidirectional(value.GetBool());
        } else if (name.Cmp(wxT("AINetworkArcSelection_IsPushBackRoute"))==0) {
                GetContent()->SetIsPushBackRoute(value.GetBool());
        } else if (name.Cmp(wxT("AINetworkArcSelection_Name"))==0) {
                GetContent()->SetName(property->GetValueAsString());
        } else {
                MapSelection::UpdateObjectFromProperty(property);
        }
}

