#include "AINetworkVerifier.h"

#include "AINetworkVertex.h"
#include "AINetworkArc.h"
#include "StartupLocationMapObject.h"

#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

#include <iostream>

AINetworkVerifier::AINetworkVerifier (Airport::SmartRef airport, MapEditor * editor):
        _airport (airport), _editor (editor)
{
}

AINetworkVerifier::~AINetworkVerifier ()
{
}

bool AINetworkVerifier::CheckDuplicateArcs ()
{
	MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
	MapObjectList objects, parkings, rwyPoints;
	MapObjectList::Node * objNode, *objNodeComp;
	AINetworkVertex::SmartRef source, target, source2, target2;
        
	MapSelectionManager & mgr = GetEditor ()->GetSelectionManager ();

	//std::cerr << "Checking for duplicate arcs" << std::endl;
	groundNetworkLayer->GetObjectsOfClass (objects, CLASSINFO (AINetworkArc));
	//groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(AINetworkNode));
	//groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(StartupLocationMapObject));
	for (objNode = objects.GetFirst (); objNode; objNode = objNode->GetNext ()) {
		AINetworkArc::SmartRef obj = (AINetworkArc *) objNode->GetData ();
		source = obj->GetSource ();
		target = obj->GetTarget ();
		objNodeComp = objNode;
		objNodeComp = objNodeComp->GetNext ();
		while (objNodeComp) {
			AINetworkArc::SmartRef obj2 = (AINetworkArc *) objNodeComp->GetData ();
			source2 = obj2->GetSource ();
			target2 = obj2->GetTarget ();
			if ((source == source2) && (target == target2)) {
				wxMessageBox (wxT ("Duplicate Taxiroute Found"));

				mgr.DeselectAll ();
				mgr.AddSelectionFor (obj2);
				return false;
			}
			objNodeComp = objNodeComp->GetNext ();
		}
	}

	return true;
}

bool AINetworkVerifier::CheckHasRunwayPoints() {
    int parkIdx = 0;

    MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
    MapObjectList  rwyPoints;
    MapObjectList::Node * objNode;

    groundNetworkLayer->GetObjectsOfClass (rwyPoints, CLASSINFO (AINetworkNode));

    for (objNode = rwyPoints.GetFirst (); objNode; objNode = objNode->GetNext ()) {
        AINetworkNode::SmartRef rwy = (AINetworkNode *) objNode->GetData ();
        if (rwy->IsOnRunway ()) {
             parkIdx++;
        }
     }
    if (parkIdx) {
       return true;
    } else {
         wxMessageBox (wxT ("Error: Ground network doesn't have OnRunway Points yet. See http://wiki.flightgear.org/index.php/Interactive_Traffic for more details"));
         return false;
    }
}

bool AINetworkVerifier::CheckParkingRunwayRouting ()
{
        MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
	MapObjectList objects, parkings, rwyPoints;
	MapObjectList::Node * objNode, *objNodeComp;
	AINetworkVertex::SmartRef source, target, source2, target2;

	MapSelectionManager & mgr = GetEditor ()->GetSelectionManager ();

        // Checking for route between parking and runway, and vice versa
	groundNetworkLayer->GetObjectsOfClass (parkings, CLASSINFO (StartupLocationMapObject));
	groundNetworkLayer->GetObjectsOfClass (rwyPoints, CLASSINFO (AINetworkNode));
	int parkIdx = 0;

	for (objNode = parkings.GetFirst (); objNode; objNode = objNode->GetNext ()) {
		parkIdx++;
		for (objNodeComp = rwyPoints.GetFirst (); objNodeComp; objNodeComp = objNodeComp->GetNext ()) {
			AINetworkNode::SmartRef rwy = (AINetworkNode *) objNodeComp->GetData ();
			if (rwy->IsOnRunway ()) {

				mgr.DeselectAll ();

				AINetworkNode::SmartRef park = (AINetworkNode *)
					objNode->GetData ();

				std::cerr << "Finding Shortest route for " << parkIdx << std::endl;
				if (!findShortestRoute (park, rwy)) {
					mgr.AddSelectionFor (park);
					mgr.AddSelectionFor (rwy);
					return false;
				}
				std::cerr << "Reverse Shortest route for " << parkIdx << std::endl;
				if (!findShortestRoute (rwy, park)) {
					mgr.AddSelectionFor (park);
					mgr.AddSelectionFor (rwy);
					return false;
				}
			}
		}
	}
        return true;
}

bool AINetworkVerifier::SetPushbackRoutes() {
	MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
	MapObjectList objects, parkings, rwyPoints;
	MapObjectList::Node * objNode, *objNodeComp;
	AINetworkVertex::SmartRef source, target, source2, target2;
        
	MapSelectionManager & mgr = GetEditor ()->GetSelectionManager ();

	groundNetworkLayer->GetObjectsOfClass (parkings, CLASSINFO (StartupLocationMapObject));
	groundNetworkLayer->GetObjectsOfClass (rwyPoints, CLASSINFO (AINetworkNode));


	// Set pushback route for each gate
        std::cerr << "Finding Pushback routes " << std::endl;
	int parkIdx = 0;
	for (objNode = parkings.GetFirst (); objNode; objNode = objNode->GetNext ()) {
		parkIdx++;
		int nRoutes = 0;
		mgr.DeselectAll ();
                std::cerr << "Scanning " << std::endl;
		for (objNodeComp = rwyPoints.GetFirst (); objNodeComp; objNodeComp = objNodeComp->GetNext ()) {
                        std::cerr << "Finding Pushback route for " << parkIdx << std::endl;
			AINetworkNode::SmartRef nde = (AINetworkNode *) objNodeComp->GetData ();
			if (nde->GetHoldPointType () == AINetworkNode::HoldPointType_PushBack) {
				StartupLocationMapObject::SmartRef park = (StartupLocationMapObject *)
					objNode->GetData ();
				std::cerr << "Finding Pushback route for " << parkIdx << std::endl;
				nRoutes += setPushbackRoute (park, nde);
				if (nRoutes > 1) {
					wxMessageBox (wxT ("Error: Startup Location has multiple push-back points"));
					mgr.AddSelectionFor (park);
					mgr.AddSelectionFor (nde);
					return false;
				}
                                //std::cerr << "Found one route " << std::endl;
			}
		}
		if (nRoutes < 1) {
			std::cerr << "No route found for " << parkIdx << std::endl;
                        //mgr.AddSelectionFor(objNode->GetData());
                        //return false;
		}
	}
        return true;
}

void
AINetworkVerifier::VerifyAINetwork ()
{
	//std::cerr << "Verifying AI Network" << std::endl;

        if (!CheckHasRunwayPoints ())
		return;
	if (!CheckDuplicateArcs ())
		return;
        if (!CheckParkingRunwayRouting())
                return;
        if (!SetPushbackRoutes())
                return;
}

bool
AINetworkVerifier::findShortestRoute (AINetworkVertex * start, AINetworkVertex * end)
{
	MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
	MapObjectList parkings, nodes, justnodes, unvisited, arcs;
	MapObjectList::Node * objNode, *best;
	//AINetworkVertex::SmartRef source,target, source2,target2;

	//groundNetworkLayer->GetObjectsOfClass(parkings, CLASSINFO(StartupLocationMapObject));
	//groundNetworkLayer->GetObjectsOfClass(justnodes, CLASSINFO(AINetworkNode));
	groundNetworkLayer->GetObjectsOfClass (nodes, CLASSINFO (AINetworkVertex));
	groundNetworkLayer->GetObjectsOfClass (arcs, CLASSINFO (AINetworkArc));
	if (start == end) {
		std::cerr << "Error: Start equals end" << std::endl;
		exit (1);
	}

	//std::cerr << "Parkings " << parkings.size() << "Nodes " << justnodes.size() << "All " << nodes.size() << std::endl;

	/*
	   for (FGTaxiNodeVectorIterator
	   itr = currNodesSet->begin();
	   itr != currNodesSet->end(); itr++) {
	   (*itr)->setPathScore(HUGE_VAL); //infinity by all practical means
	   (*itr)->setPreviousNode(0); //
	   (*itr)->setPreviousSeg (0); //
	   }
	 */
	for (objNode = nodes.GetFirst (); objNode; objNode = objNode->GetNext ()) {
		AINetworkVertex::SmartRef park = (AINetworkVertex *) objNode->GetData ();
		park->setPathScore (HUGE);
		park->setPreviousNode (0);
		park->setPreviousArc (0);
	}
	/* 
	   FGTaxiNode *firstNode = findNode(start);
	   firstNode->setPathScore(0);
	 */
	start->setPathScore (0);

	/* FGTaxiNode *lastNode  = findNode(end); */

	/*FGTaxiNodeVector unvisited(*currNodesSet); // working copy */
	unvisited = nodes;

	/*
	   while (!unvisited.empty()) {
	   FGTaxiNode* best = *(unvisited.begin());
	   for (FGTaxiNodeVectorIterator
	   itr = unvisited.begin();
	   itr != unvisited.end(); itr++) {
	   if ((*itr)->getPathScore() < best->getPathScore())
	   best = (*itr);
	   }
	 */
	while (unvisited.size ()) {
		//std::cerr << unvisited.size() << "Nodes to process" << std::endl;
		best = unvisited.GetFirst ();
		AINetworkVertex::SmartRef bestNode = (AINetworkVertex *) best->GetData ();
		for (objNode = unvisited.GetFirst (); objNode; objNode = objNode->GetNext ()) {
			AINetworkVertex::SmartRef currNode = (AINetworkVertex *) objNode->GetData ();
			//std::cerr << "Path Score : " << currNode->getPathScore() << std::endl;
			if (currNode->getPathScore () < bestNode->getPathScore ()) {
				best = objNode;
				//std::cerr << "Found best path : " << currNode->getPathScore() << std::endl;
			}
			if (currNode == start) {
				//std::cerr << "Current node = start! " << start->getPathScore() << " " << currNode->getPathScore()
				//     << std::endl;
			}
		}
		bestNode = (AINetworkVertex *) best->GetData ();

		/*
		   if (best == lastNode) { // found route or best not connected
                           break;
		   } else {
                           for (FGTaxiSegmentVectorIterator
                                   seg = best->getBeginRoute();
                                   seg != best->getEndRoute(); seg++) {
                                if (fullSearch || (*seg)->isPushBack()) {
                                        FGTaxiNode* tgt = (*seg)->getEnd();
                                        double alt = best->getPathScore() + (*seg)->getLength() + (*seg)->getPenalty(nParkings);
                                        if (alt < tgt->getPathScore()) {              // Relax (u,v)
                                                tgt->setPathScore(alt);
                                                tgt->setPreviousNode(best);
                                                tgt->setPreviousSeg(*seg); //
                                        }
                                } else {
                                        //   // std::cerr << "Skipping TaxiSegment " << (*seg)->getIndex() << std::endl;
                                }
                           }
                   }
		 */

		if (bestNode == end) {	// found route or best not connected
			//std::cerr << "Best node is end "<< std::endl;
			break;
		}
		else {
			//AINetworkVertexArcs *objArc;// = bestNode->getArcs();
			wxAINetworkVertexArcsNode *objArc;
			//std::cerr << "Scanning" << std::endl;
			for (objArc = bestNode->getFirstArc (); objArc; objArc = objArc->GetNext ()) {

				AINetworkArc::SmartRef arcData = (AINetworkArc *)
					objArc->GetData ();
				AINetworkVertex *target = arcData->GetTarget ();
				AINetworkVertex *source = arcData->GetSource ();
				//FIXME: Remove before Flight:
				if ((source != bestNode)
				    && (target != bestNode)) {
					std::cerr << "Neither source nor target match: Arc seems disconnected" << std::endl;
					exit (1);
				}
				if (target == bestNode) {
					//std::cerr << "Current node matches target value : ";
					if (arcData->IsBidirectional ()) {
						//std::cerr << "BiDirectional, using source instead" << std::endl;
						target = source;
					}
					else {
						//std::cerr << "Is unidirectional. Skipping arc" << std::endl;
						target = 0;
					}
				}
				if (target) {
					//std::cerr << "We have a target" << std::endl;
					//TODO: Since we're only interested in whether a path acually
					// exists, and not so much in finding the most optimum route,
					// just set path score to 1. here. We might want to change that to 
					// something more sensible later on.
					double alt = bestNode->getPathScore () + 1.0;
					//std::cerr << "Current path score " << alt << "Target path" << target->getPathScore() << std::endl;
					if (alt < target->getPathScore ()) {
						target->setPathScore (alt);
						target->setPreviousNode (bestNode);
						target->setPreviousArc (arcData);
						//std::cerr << "Found better path score " << alt;
						//if (target == end) {
						//   std::cerr << "Found route" << std::endl;
						//}
					}
				}

			}
		}
		unvisited.Erase (best);
	}
	//if (lastNode->getPathScore() == HUGE_VAL) {
	if (end->getPathScore () == HUGE) {
		wxMessageBox (wxT ("No Route Found"));
		return false;

	}
	else {
		return true;
	}
}

int
AINetworkVerifier::setPushbackRoute (StartupLocationMapObject * start, AINetworkVertex * end)
{
	MapObjectContainer::SmartRef groundNetworkLayer = GetAirport ()->GetGroundNetworkLayer ();
	MapObjectList parkings, nodes, justnodes, unvisited, arcs;
	MapObjectList::Node * objNode, *best;
	//AINetworkVertex::SmartRef source,target, source2,target2;

	//groundNetworkLayer->GetObjectsOfClass(parkings, CLASSINFO(StartupLocationMapObject));
	//groundNetworkLayer->GetObjectsOfClass(justnodes, CLASSINFO(AINetworkNode));
	groundNetworkLayer->GetObjectsOfClass (nodes, CLASSINFO (AINetworkVertex));
	groundNetworkLayer->GetObjectsOfClass (arcs, CLASSINFO (AINetworkArc));
	if (start == end) {
		std::cerr << "Error: Start equals end" << std::endl;
		exit (1);
	}
	/* 
	   objArc=arcs.GetFirst();
	   while (objArc)  {
	   AINetworkArc::SmartRef arcData = (AINetworkArc*) objArc->GetData();
	   if (arcData->IsPushBackRoute()) {
	   objArc=objArc->GetNext();
	   } else {
	   arcs.Erase(objArc);
	   objArc=arcs.GetFirst();
	   }
	   } */

	for (objNode = nodes.GetFirst (); objNode; objNode = objNode->GetNext ()) {
		AINetworkVertex::SmartRef park = (AINetworkVertex *) objNode->GetData ();
		park->setPathScore (HUGE);
		park->setPreviousNode (0);
		park->setPreviousArc (0);
	}
	start->setPathScore (0);

	unvisited = nodes;	// Create a working copy

	while (unvisited.size ()) {
		best = unvisited.GetFirst ();
		AINetworkVertex::SmartRef bestNode = (AINetworkVertex *) best->GetData ();
		for (objNode = unvisited.GetFirst (); objNode; objNode = objNode->GetNext ()) {
			AINetworkVertex::SmartRef currNode = (AINetworkVertex *) objNode->GetData ();
			if (currNode->getPathScore () < bestNode->getPathScore ()) {
				best = objNode;
			}
		}
		bestNode = (AINetworkVertex *) best->GetData ();
		if (bestNode == end) {	// found route or best not connected
			//std::cerr << "Best node is end "<< std::endl;
			break;
		}
		else {
			//AINetworkVertexArcs *objArc;// = bestNode->getArcs();
			wxAINetworkVertexArcsNode *objArc;
			//std::cerr << "Scanning" << std::endl;
			for (objArc = bestNode->getFirstArc (); objArc; objArc = objArc->GetNext ()) {

				AINetworkArc::SmartRef arcData = (AINetworkArc *)
					objArc->GetData ();
				AINetworkVertex *target = arcData->GetTarget ();
				AINetworkVertex *source = arcData->GetSource ();
				//FIXME: Remove before Flight:
				if ((source != bestNode)
				    && (target != bestNode)) {
					std::cerr << "Neither source nor target match: Arc seems disconnected" << std::endl;
					exit (1);
				}
				if (target == bestNode) {
					//std::cerr << "Current node matches target value : ";
					if (arcData->IsBidirectional ()) {
						//std::cerr << "BiDirectional, using source instead" << std::endl;
						target = source;
					}
					else {
						//std::cerr << "Is unidirectional. Skipping arc" << std::endl;
						target = 0;
					}
				}
				if (!(arcData->IsPushBackRoute ())) {
					target = 0;
				}
				if (target) {
					//std::cerr << "We have a target" << std::endl;
					//TODO: Since we're only interested in whether a path acually
					// exists, and not so much in finding the most optimum route,
					// just set path score to 1. here. We might want to change that to 
					// something more sensible later on.
					double alt = bestNode->getPathScore () + 1.0;
					//std::cerr << "Current path score " << alt << "Target path" << target->getPathScore() << std::endl;
					if (alt < target->getPathScore ()) {
						target->setPathScore (alt);
						target->setPreviousNode (bestNode);
						target->setPreviousArc (arcData);
						//std::cerr << "Found better path score " << alt;
						//if (target == end) {
						//   std::cerr << "Found route" << std::endl;
						//}
					}
				}

			}
		}
		unvisited.Erase (best);
	}
	//if (lastNode->getPathScore() == HUGE_VAL) {
	if (end->getPathScore () == HUGE) {
		//wxMessageBox(wxT("No Route Found"));
		return 0;
	}
	else {
		/*
		   intVec nodes, routes;
		   FGTaxiNode* bt = lastNode;
		   while (bt->getPreviousNode() != 0) {
		   nodes.push_back(bt->getIndex());
		   routes.push_back(bt->getPreviousSegment()->getIndex());
		   bt = bt->getPreviousNode();
		   } 
		   nodes.push_back(start);
		 */
                std::cerr << "Success " << std::endl;
		start->clearPushBackRoute ();
		AINetworkVertex *target = end;
		//while (target->getPreviousNode()) {
		start->InsertPushbackPoint (target);
		//}
		return 1;
	}
}
