#ifndef StartupLocationMapObject_h
#define StartupLocationMapObject_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "AINetworkVertex.h"
#include "AINetworkNode.h"
#include <Transactions/TransactionObjectChangeListener.h>

#include <Airport/Airport.h>

WX_DECLARE_LIST(AINetworkVertex,AINetworkVertexNodes);

class StartupLocationMapObject: public AINetworkVertex,
                                public TransactionObjectChangeListener {
        
        DECLARE_CLASS(StartupLocationMapObject)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(StartupLocationMapObject);
        
        friend class StartupLocationSelection;
        
public:
        typedef enum {
                GateType_Gate,
                GateType_Cargo,
                GateType_GA,
                GateType_Mil_Fighter,
                GateType_Mil_Cargo
        } GateType;
        
	StartupLocationMapObject(Airport::SmartRef airport,
                                 Point3D position,
                                 float heading,
                                 wxString gatename,
                                 wxString gatenumber=wxT(""),
                                 GateType gatetype=GateType_Gate,
                                 wxString airlinecodes=wxT(""),
                                 float radius=17.3 /* Standard for Airbus A320 */);
        
        virtual ~StartupLocationMapObject();
        
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        virtual bool Hit(MapEditor* editor, TDRectangle r);
        
        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        virtual TDRectangle GetGeographicBounds();
        
        const wxString GetName() const {
                return GetGateName()+GetGateNumber();
        }

        void clearPushBackRoute();
        void InsertPushbackPoint(AINetworkVertex*);
        wxAINetworkVertexNodesNode *getFirstPushBackPoint();
        bool hasPushbackRoute() { return (bool) pushBackNodes.size(); };
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(float,Heading,_heading)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(wxString,GateName,_gatename,
                                                TRANSACTIONOBJECT_STATEVAR(_gatename).Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(wxString,GateNumber,_gatenumber,
                                                TRANSACTIONOBJECT_STATEVAR(_gatenumber).Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(float,Radius,_radius)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(wxString,AirlineCodes,_airlinecodes,
                                                TRANSACTIONOBJECT_STATEVAR(_airlinecodes).Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(GateType,GateType,_type)
        Airport::SmartRef GetAirport() const { return _airport; }
        
        virtual void BeforeObjectChange(TransactionObject* object) {
                if (object==_airport) {
                        Damage();
                }
        }
        
        virtual void AfterObjectChange(TransactionObject* object) {
                if (object==_airport) {
                        Damage();
                }
        }
protected:
        typedef struct {
                float _heading;
                /* Ground network stuff */
                wxString _gatename;
                wxString _gatenumber;
                GateType _type;
                wxString _airlinecodes;
                float _radius;
        } State;

        AINetworkVertexNodes pushBackNodes;

        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(StartupLocationMapObject,AINetworkVertex,State)
protected:
        Airport::SmartRef _airport;
        void GetLocalCornerPoints(MapEditor* editor, Point3D pts[4]);
        
};

#endif

