#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "TaxiwayMapObject.h"

#include "TaxiwaySelection.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(TaxiwayMapObject,PavementMapObject)

TaxiwayMapObject::TaxiwayMapObject(Airport::SmartRef airport,
                                   Point3D center,
                                   double length,
                                   double width,
                                   double heading,
                                   Surface surface,
                                   float smoothness,
                                   bool edgeLights,
                                   bool centerLights):
        PavementMapObject(airport,center,length,width,heading,surface,smoothness)
{
        _myState.hasEdgeLights=edgeLights;
        _myState.hasCenterLights=centerLights;
}

TaxiwayMapObject::~TaxiwayMapObject() {
}

void TaxiwayMapObject::Paint(wxDC& dc, MapEditor* editor, const MapPass& pass) {
        PavementMapObject::Paint(dc,editor,pass);

        MapProjection* proj=editor->GetProjection();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        
        const Point3D* centerEnds=GetEndCenters();
        Point3D localCenterEnds[2];
        wxPoint devCenterEnds[2];
        for (int i=0;i<2;i++) {
                localCenterEnds[i]=proj->ConvertToLocal(centerEnds[i]);
                devCenterEnds[i]=cs.LogicalToDevice(localCenterEnds[i]);
        }

        if (pass==LightingPass) {
                wxPen centerLightsPen(wxColour(0, 255, 0), 3, wxDOT);
                wxPen edgeLightsPen(wxColour(0, 0, 255), 3, wxDOT);

                if (HasCenterLights()) {
                        
                        dc.SetPen(centerLightsPen);
                        
                        dc.DrawLine(devCenterEnds[0],devCenterEnds[1]);
                        
                        dc.SetPen(wxNullPen);
                }
                
                if (HasEdgeLights()) {
                        const Point3D* corners=GetEndCorners();
                        Point3D localCorners[4];
                        wxPoint devCorners[4];
                        for (int i=0;i<4;i++) {
                                localCorners[i]=proj->ConvertToLocal(corners[i]);
                                devCorners[i]=cs.LogicalToDevice(localCorners[i]);
                        }
                        
                        dc.SetPen(edgeLightsPen);
                        
                        dc.DrawLine(devCorners[1],devCorners[2]);
                        dc.DrawLine(devCorners[0],devCorners[3]);
                        
                        dc.SetPen(wxNullPen);
                }
        }
}

bool TaxiwayMapObject::Hit(MapEditor* editor, TDRectangle r) {
        // TODO: Remove this method or implement your own
        return PavementMapObject::Hit(editor,r);
}

MapSelection* TaxiwayMapObject::CreateSelection(MapSelectionManager* manager) {
        return new TaxiwaySelection(manager,this);
}

wxRect TaxiwayMapObject::GetDeviceBounds(MapEditor* editor) {
        wxRect r=PavementMapObject::GetDeviceBounds(editor);
        /*
         * if edge lights are enabled, make sure that the
         * light lines (width=3) are included.
         */
        if (HasEdgeLights()) {
                r.Inflate(3,3);
        }
        
        return r;
}

TDRectangle TaxiwayMapObject::GetGeographicBounds() {
        // TODO: Remove this method or implement your own
        return PavementMapObject::GetGeographicBounds();
}


