#include "OldProjectWriter.h"

#include "XPlaneWriter.h"

#include <Imagery/MapCalibratedBackgroundImage.h>
#include <Map/MapObjectContainer.h>
#include <Map/MapProjection.h>
#include <Map/MapProjectionDescriptor.h>

IMPLEMENT_CLASS(OldProjectWriter,wxObject);

OldProjectWriter::OldProjectWriter(std::ostream& outfile):
        _outfile(outfile)
{
}

OldProjectWriter::~OldProjectWriter() {
}

void OldProjectWriter::writeProject(Airport::SmartRef airport,
                                    MapProjection* projection,
                                    MapObjectContainer* backgroundImageLayer)
{
        _outfile << "[FORMAT]\n";
        _outfile << "X-PLANE 810\n";
        _outfile << "[END-FORMAT]\n";
        
        MapObjectList bgndImages;
        backgroundImageLayer->GetObjectsOfClass(bgndImages,CLASSINFO(MapCalibratedBackgroundImage));
        MapObjectList::Node *objNode;
        
        if (bgndImages.GetCount()!=0) {
                for (objNode=bgndImages.GetFirst();objNode;objNode=objNode->GetNext()) {
                    writeCalibration(wxDynamicCast(objNode->GetData(),MapCalibratedBackgroundImage),
                                     projection);
                }
        }
        
        writeAirport(airport);
}

void OldProjectWriter::writeCalibration(MapCalibratedBackgroundImage::SmartRef backgroundImage,
                                        MapProjection* projection)
{
        double scale,angle;
        Point3D pos;
        
        scale=backgroundImage->GetScale();
        angle=backgroundImage->GetAngle();
        pos=backgroundImage->GetNorthWestPos();
        
        _outfile << "[CALIBRATION]\n";
        _outfile << "SCALE " << scale << "\n";
        _outfile << "ANGLE " << angle << "\n";
        _outfile << "LAT " << pos.lat() << "\n";
        _outfile << "LON " << pos.lon() << "\n";
        _outfile << "FILE " << backgroundImage->GetFilename().mb_str(wxConvUTF8) << "\n";
        _outfile << "PROJECTION " << projection->GetDescriptor()->GetName().mb_str(wxConvUTF8) << "\n";
        _outfile << "[END-CALIBRATION]\n";
}

void OldProjectWriter::writeAirport(Airport::SmartRef airport)
{
        _outfile << "[AIRPORT]\n";
        XPlaneWriter writer(_outfile);
        writer.setVersion(810);
        writer.writeAirport(airport);
}

