#ifndef XPlaneWriter_h
#define XPlaneWriter_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Airport/Airport.h>
#include <Airport/BeaconMapObject.h>
#include <Airport/PavementMapObject.h>
#include <Airport/RunwayMapObject.h>
#include <Airport/TowerMapObject.h>
#include <Airport/TaxiwayMapObject.h>
#include <Airport/WindsockMapObject.h>

#include <AI/StartupLocationMapObject.h>

class XPlaneWriter: public wxObject {
        
        DECLARE_CLASS(XPlaneWriter)
        
public:
	XPlaneWriter(std::ostream& outfile);
        virtual ~XPlaneWriter();
        
        void setVersion(int version) {
                _version=version;
        }
        
        void writeHeader();
        void writeAirport(Airport::SmartRef airport);
        void writeFooter();
        
        void mergeAirport(std::istream& infile, Airport::SmartRef airport);
protected:
        void writeAirportHeader(Airport::SmartRef airport);
        void writeRunway(RunwayMapObject::SmartRef object);
        void writeTaxiway(TaxiwayMapObject::SmartRef twyObject);
        void writeTowerViewpoint(TowerMapObject::SmartRef obj);
        void writeStartupLocation(StartupLocationMapObject::SmartRef obj);
        void writeBeacon(BeaconMapObject::SmartRef obj);
        void writeWindsock(WindsockMapObject::SmartRef obj);
        void writeFrequency(FrequencyEntry freq);
        
        int GetCodeForVAPL(RunwayMapObject::VisualApproachPathLighting vapl);
        int GetCodeForRWYL(RunwayMapObject::RunwayLighting rwyl);
        int GetCodeForAPPL(RunwayMapObject::ApproachLighting appl);
        int GetCodeForShoulder(RunwayMapObject::Shoulder shoulder);
        int GetCodeForMarkings(RunwayMapObject::Markings markings);
        
        int GetCodeForSurface(PavementMapObject::Surface surface);
        
        std::ostream& _outfile;
        
        int _version;
};

#endif

