#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapFrame.h"
#include "MapObjectPlacer.h"
#include "MapPropertiesFrame.h"
#include "MapStatusBar.h"
#include "MapViewManager.h"
#include "PreferenceDialog.h"
#include "ProjectWizard.h"

#include <AI/AIGroundNetworkLoader.h>
#include <AI/AIGroundNetworkWriter.h>
#include <AI/AINetworkNodePlacementMode.h>
#include <AI/AINetworkVerifier.h>
#include <AI/AINetworkVertex.h>
#include <AI/StartupLocationMapObjectPlacer.h>



#include <Airport/Airport.h>
#include <Airport/BeaconMapObjectPlacer.h>
#include <Airport/TowerMapObjectPlacer.h>
#include <Airport/WindsockMapObjectPlacer.h>

#include <Imagery/CalibrationMode.h>
#include <Imagery/MapUSGSImageLayer.h>
#include <Imagery/USGSFetcher.h>
#include <Imagery/USGSLoadMode.h>

#include <IO/OldProjectWriter.h>
#include <IO/XPlaneLoader.h>
#include <IO/XPlaneWriter.h>

#include <Map/MapEditor.h>
#include <Map/MapGrid.h>
#include <Map/MapObjectContainer.h>
#include <Map/MapProjectionMenuManager.h>
#include <Map/ModeManager.h>

#include <Modes/CoordinateDisplayMode.h>
#include <Modes/ObjectPlacementMode.h>
#include <Modes/PanZoomMode.h>
#include <Modes/PavementInsertionMode.h>
#include <Modes/PopupMode.h>
#include <Modes/SelectionMode.h>

#include <Projection/UTMProjection.h>

#include <Transactions/Transaction.h>
#include <Transactions/TransactionManager.h>
#include <Transactions/UndoRedoMenuManager.h>

#include <xml/easyxml.hxx>

#include "icons/icon_new_png.h"
#include "icons/icon_open_png.h"
#include "icons/icon_save_png.h"
#include "icons/icon_import_png.h"
#include "icons/icon_export_png.h"
#include "icons/icon_import_ai_png.h"
#include "icons/icon_export_ai_png.h"
#include "icons/icon_undo_png.h"
#include "icons/icon_redo_png.h"

#include "icons/icon_showhide_png.h"
#include "icons/icon_lock_png.h"
#include "icons/icon_zoom_png.h"
#include "icons/icon_unzoom_png.h"
#include "icons/icon_zoom_full_png.h"
#include "icons/icon_zoom_sel_png.h"
#include "icons/icon_zoom_reset_png.h"

#include "icons/icon_selection_png.h"
#include "icons/icon_runway_png.h"
#include "icons/icon_taxiway_png.h"
#include "icons/icon_sign_png.h"
#include "icons/icon_beacon_png.h"
#include "icons/icon_startuppos_png.h"
#include "icons/icon_tower_png.h"
#include "icons/icon_windsock_png.h"
#include "icons/icon_ainodes_png.h"
#include "icons/icon_aiarcs_uni_png.h"
#include "icons/icon_aiarcs_bi_png.h"
#include "icons/icon_usgs_png.h"

#include "icons/icon_16.xpm"
#include "icons/icon_24.xpm"
#include "icons/icon_32.xpm"
#include "icons/icon_48.xpm"

#include <wx/config.h>
#include <wx/features.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/mstream.h>
#include <wx/progdlg.h>
#include <wx/scrolwin.h>
#include <wx/textdlg.h>

#include <wx/menutoolbar/menutoolbar.h>

#include <fstream>

#define wxGetBitmapFromMemory(name) _wxGetBitmapFromMemory(name ## _png, sizeof(name ## _png))


inline wxBitmap _wxGetBitmapFromMemory(const unsigned char *data, int length) {
        wxMemoryInputStream is(data, length);
        return wxBitmap(wxImage(is, wxBITMAP_TYPE_ANY, -1), -1);
}

IMPLEMENT_CLASS(MapFrame,wxFrame);

enum {
        MapFrame_ID_Actions=wxID_HIGHEST,
        
        MapFrame_ID_ImportAirport,
        MapFrame_ID_ExportAirport,
        MapFrame_ID_ImportAINetwork,
        MapFrame_ID_ExportAINetwork,
        
        MapFrame_ID_UndoMenu,
        MapFrame_ID_RedoMenu,
        
        MapFrame_ID_DeselectAll,
        
        MapFrame_ID_ModesMenu,
        MapFrame_ID_ActionsMenu,

        MapFrame_ID_SelectionMode,
        MapFrame_ID_RunwayInsertionMode,
        MapFrame_ID_TaxiwayInsertionMode,
        //MapFrame_ID_SignInsertionMode,
        MapFrame_ID_BeaconInsertionMode,
        MapFrame_ID_StartupLocationInsertionMode,
        MapFrame_ID_TowerViewpointInsertionMode,
        MapFrame_ID_WindsockInsertionMode,
        MapFrame_ID_AINodeInsertionMode,
        MapFrame_ID_AIArcingMode_Unidirectional,
        MapFrame_ID_AIArcingMode_Bidirectional,
        MapFrame_ID_USGSLoadMode,
        MapFrame_ID_LASTMODE,
        MapFrame_ID_VerifyGroundNetwork,
        
        MapFrame_ID_NewView,
        MapFrame_ID_ShowHideLayers,
        MapFrame_ID_LockUnlockLayers,
        MapFrame_ID_ShowProperties,
        MapFrame_ID_ToggleGrid,
        MapFrame_ID_ZoomIn,
        MapFrame_ID_ZoomOut,
        MapFrame_ID_ZoomReset,
        MapFrame_ID_ZoomFullView,
        MapFrame_ID_ZoomToSelection,
        MapFrame_ID_ScrollUp,
        MapFrame_ID_ScrollDown,
        MapFrame_ID_ScrollLeft,
        MapFrame_ID_ScrollRight,
        MapFrame_ID_LoadBackgroundImage,
        MapFrame_ID_ImageryTheme,
        MapFrame_ID_ImageryTheme_DOQ,
        MapFrame_ID_ImageryTheme_DRG,
        MapFrame_ID_ImageryTheme_Urban,
        MapFrame_ID_ImageryRes,
        MapFrame_ID_ImageryRes_0,
        MapFrame_ID_ImageryRes_19=MapFrame_ID_ImageryRes_0+19,
        
        MapFrame_ID_FlipObjectsOrder,
        
        MapFrame_ID_LASTEVENT
};

BEGIN_EVENT_TABLE(MapFrame, wxFrame)
        // menus
        EVT_MENU_OPEN(MapFrame::OnMenuOpen)
        EVT_MENU(wxID_NEW,                   MapFrame::OnNewProject)
        EVT_MENU(wxID_OPEN,                  MapFrame::OnOpenProject)
        EVT_MENU(wxID_SAVE,                  MapFrame::OnSaveProject)
        EVT_MENU(wxID_SAVEAS,                MapFrame::OnSaveProject)
        EVT_MENU(MapFrame_ID_ImportAirport,  MapFrame::OnImportAirport)
        EVT_MENU(MapFrame_ID_ExportAirport,  MapFrame::OnExportAirport)
        EVT_MENU(MapFrame_ID_ImportAINetwork,MapFrame::OnImportAINetwork)
        EVT_MENU(MapFrame_ID_ExportAINetwork,MapFrame::OnExportAINetwork)
        EVT_MENU(wxID_CLOSE,                 MapFrame::OnClose)
        EVT_MENU(wxID_EXIT,                  MapFrame::OnExit)
        
        EVT_MENU(wxID_UNDO,                  MapFrame::OnUndo)
        EVT_MENU(wxID_REDO,                  MapFrame::OnRedo)
        EVT_MENU(wxID_DELETE,                MapFrame::OnDelete)
        EVT_MENU(MapFrame_ID_DeselectAll,    MapFrame::OnDeselectAll)
        EVT_MENU(wxID_PROPERTIES,            MapFrame::OnPreferences)
        
        EVT_MENU(MapFrame_ID_NewView,        MapFrame::OnNewView)
        EVT_MENU(MapFrame_ID_ShowProperties, MapFrame::OnShowProperties)
        EVT_MENU(MapFrame_ID_ToggleGrid,     MapFrame::OnToggleGrid)
        EVT_MENU(MapFrame_ID_ZoomIn,         MapFrame::OnZoomIn)
        EVT_MENU(MapFrame_ID_ZoomOut,        MapFrame::OnZoomOut)
        EVT_MENU(MapFrame_ID_ZoomReset,      MapFrame::OnZoomReset)
        EVT_MENU(MapFrame_ID_ZoomFullView,   MapFrame::OnZoomFullView)
        EVT_MENU(MapFrame_ID_ZoomToSelection,MapFrame::OnZoomToSelection)
        
        EVT_MENU(MapFrame_ID_ScrollUp,       MapFrame::OnScroll)
        EVT_MENU(MapFrame_ID_ScrollDown,     MapFrame::OnScroll)
        EVT_MENU(MapFrame_ID_ScrollLeft,     MapFrame::OnScroll)
        EVT_MENU(MapFrame_ID_ScrollRight,    MapFrame::OnScroll)

        EVT_MENU(MapFrame_ID_LoadBackgroundImage,MapFrame::OnLoadBackgroundImage)

        EVT_MENU(MapFrame_ID_VerifyGroundNetwork,MapFrame::OnVerifyGroundNetwork)
        
        EVT_MENU_RANGE(MapFrame_ID_SelectionMode,MapFrame_ID_LASTMODE,MapFrame::OnMode)
        
        EVT_MENU_RANGE(MapFrame_ID_ImageryTheme_DOQ,MapFrame_ID_ImageryTheme_Urban,MapFrame::OnImageryThemeType)
        EVT_MENU_RANGE(MapFrame_ID_ImageryRes_0,MapFrame_ID_ImageryRes_19,MapFrame::OnImageryResolution)
        
        EVT_MENU(MapFrame_ID_FlipObjectsOrder, MapFrame::OnFlipObjectsOrder)
        
        EVT_KEY_DOWN(MapFrame::OnKeyDown)
        
        EVT_COMMAND(MapFrame_ID_UndoMenu, wxEVT_FILL_MENU, MapFrame::OnFillUndoMenu)
        EVT_COMMAND(MapFrame_ID_RedoMenu, wxEVT_FILL_MENU, MapFrame::OnFillRedoMenu)
        
        EVT_ACTIVATE(MapFrame::OnActivate)
        EVT_ICONIZE(MapFrame::OnIconize)
        
        EVT_CLOSE(MapFrame::OnCloseWindow)
END_EVENT_TABLE()

MapFrame::MapFrame(Airport* airport,
                   MapObjectContainer* backgroundImageLayer,
                   wxWindow* parent,
                   wxWindowID id,
                   const wxString& title,
                   const wxPoint& pos,
                   const wxSize& size,
                   long style,
                   const wxString& name)
   :wxFrame(parent,id,title,pos,size,style,name),
    _airport(airport),
    _backgroundImageLayer(backgroundImageLayer),
    _usgsImageLayer(new MapUSGSImageLayer(airport->GetTransactionManager(),wxT("USGS Images"))),
    _showPropertyFrameOnActivation(true)
{
        ::wxRegisterId(MapFrame_ID_LASTEVENT);
        
        SetTitle(wxT("TaxiDraw: ")+airport->GetICAOID()+wxT(" - ")+airport->GetName());
        
        wxIconBundle iconBundle;
        
        iconBundle.AddIcon(wxICON(icon_16));
        iconBundle.AddIcon(wxICON(icon_24));
        iconBundle.AddIcon(wxICON(icon_32));
        iconBundle.AddIcon(wxICON(icon_48));
        
        SetIcons(iconBundle);
        
        _statusBar=new MapStatusBar(this,wxID_ANY);
        _statusBar->SetProgressLabel(wxT("USGS images:"));
        SetStatusBar(_statusBar);
        
        SetupDrawingWindow();
        
        // The map editor depends on the drawing window being properly set up
        SetupMapEditor();
        
        // Menu bar and toolbar depend on the map editor being properly set up
        SetupMenubar();
        SetupToolbar();
        
        // So does the property frame
        _propertyFrame=new MapPropertiesFrame(this,
                                              wxID_ANY,
                                              _mapEditor,GetAirport(),
                                              wxT("Properties"));
        
        _toolbar->ToggleTool(MapFrame_ID_SelectionMode,true);
        GetMenuBar()->Check(MapFrame_ID_SelectionMode,true);
        
        USGSFetcher::GetInstance()->RegisterProgressListener(this);
}

void MapFrame::SetupMenubar() {
        wxMenuBar* menuBar=new wxMenuBar(wxMB_DOCKABLE);
        
        _fileMenu=new wxMenu();
        _fileMenu->Append(wxID_NEW, wxT("New Project...\tCTRL+N"));
        _fileMenu->Append(wxID_OPEN, wxT("Open Project...\tCTRL+O"));
        _fileMenu->Append(wxID_SAVE, wxT("Save Project\tCTRL+S"));
        _fileMenu->Append(wxID_SAVEAS, wxT("Save Project as..."));
        _fileMenu->Append(MapFrame_ID_ImportAirport, wxT("Import Airport..."));
        _fileMenu->Append(MapFrame_ID_ExportAirport, wxT("Export Airport..."));
        _fileMenu->Append(MapFrame_ID_ImportAINetwork, wxT("Import AI network..."));
        _fileMenu->Append(MapFrame_ID_ExportAINetwork, wxT("Export AI network..."));
        _fileMenu->Append(wxID_CLOSE, wxT("Close\tCTRL+W"));
        _fileMenu->Append(wxID_EXIT, wxT("Exit\tCTRL+Q"));
        menuBar->Append(_fileMenu,wxT("&File"));
        
        wxMenu *modeMenu=new wxMenu();
        modeMenu->AppendRadioItem(MapFrame_ID_SelectionMode, wxT("Selection mode"));
        modeMenu->AppendRadioItem(MapFrame_ID_RunwayInsertionMode, wxT("Insert Runway\tCTRL+SHIFT+R"));
        modeMenu->AppendRadioItem(MapFrame_ID_TaxiwayInsertionMode, wxT("Insert Taxiway\tCTRL+SHIFT+T"));
        //modeMenu->AppendRadioItem(MapFrame_ID_SignInsertionMode, wxT("Insert Sign"));
        modeMenu->AppendRadioItem(MapFrame_ID_BeaconInsertionMode, wxT("Insert Beacon\tCTRL+SHIFT+B"));
        modeMenu->AppendRadioItem(MapFrame_ID_StartupLocationInsertionMode, wxT("Insert Startup Location\tCTRL+SHIFT+P"));
        modeMenu->AppendRadioItem(MapFrame_ID_TowerViewpointInsertionMode, wxT("Insert Tower Viewpoint\tCTRL+SHIFT+V"));
        modeMenu->AppendRadioItem(MapFrame_ID_WindsockInsertionMode, wxT("Insert Windsock"));
        modeMenu->AppendRadioItem(MapFrame_ID_AINodeInsertionMode, wxT("Insert AI Nodes"));
        modeMenu->AppendRadioItem(MapFrame_ID_AIArcingMode_Unidirectional, wxT("Insert unidirectionally connected AI Nodes"));
        modeMenu->AppendRadioItem(MapFrame_ID_AIArcingMode_Bidirectional, wxT("Insert bidirectionally connected AI Nodes\tCTRL+SHIFT+A"));
        modeMenu->AppendRadioItem(MapFrame_ID_USGSLoadMode, wxT("Load USGS Images"));
        
        _actionsMenu=new wxMenu();
        _actionsMenu->SetEventHandler(this);
        _actionsMenu->SetClientData(GetEditor());

        _editMenu=new wxMenu();
        _editMenu->Append(wxID_UNDO, wxT("Undo\tCTRL+Z"));
        _editMenu->Append(wxID_REDO, wxT("Redo\tCTRL+R"));
        _editMenu->AppendSeparator();
        _editMenu->Append(wxID_DELETE, wxT("Delete\tDEL"));
        _editMenu->Append(MapFrame_ID_DeselectAll, wxT("Deselect All\tCTRL+A"));
        _editMenu->Append(MapFrame_ID_ModesMenu,wxT("Modes"),modeMenu);
        _editMenu->Append(MapFrame_ID_ActionsMenu,wxT("Actions"),_actionsMenu);
        _editMenu->AppendSeparator();
        _editMenu->Append(wxID_PROPERTIES, wxT("Preferences..."));
        menuBar->Append(_editMenu,wxT("&Edit"));
        
        _viewMenu=new wxMenu();
        _viewMenu->Append(MapFrame_ID_NewView, wxT("Open new View"));
        _viewMenu->AppendSeparator();
        _viewMenu->AppendCheckItem(MapFrame_ID_ShowProperties, wxT("Show Properties\tCTRL+P"));
        _viewMenu->Check(MapFrame_ID_ShowProperties,true);
        _viewMenu->AppendCheckItem(MapFrame_ID_ToggleGrid, wxT("View Grid\tCTRL+G"));
        _viewMenu->Check(MapFrame_ID_ToggleGrid,true);
        _viewMenu->AppendSeparator();
        _viewMenu->Append(MapFrame_ID_ZoomIn, wxT("Zoom in\tCTRL++"));
        _viewMenu->Append(MapFrame_ID_ZoomOut, wxT("Zoom out\tCTRL+-"));
        _viewMenu->Append(MapFrame_ID_ZoomReset, wxT("Reset zoom (1:1)\tCTRL+1"));
        _viewMenu->Append(MapFrame_ID_ZoomFullView, wxT("Zoom to full view\tHOME"));
        _viewMenu->Append(MapFrame_ID_ZoomToSelection, wxT("Zoom to selection\tSHIFT+HOME"));
        _viewMenu->AppendSeparator();
        _viewMenu->Append(MapFrame_ID_ScrollUp, wxT("Scroll up\tCTRL+UP"));
        _viewMenu->Append(MapFrame_ID_ScrollDown, wxT("Scroll down\tCTRL+DOWN"));
        _viewMenu->Append(MapFrame_ID_ScrollLeft, wxT("Scroll left\tCTRL+LEFT"));
        _viewMenu->Append(MapFrame_ID_ScrollRight, wxT("Scroll right\tCTRL+RIGHT"));
        _viewMenu->AppendSeparator();
        _viewMenu->Append(MapFrame_ID_LoadBackgroundImage, wxT("Load Background Image"));
        
        wxMenu* imageryThemeMenu=new wxMenu();
        imageryThemeMenu->AppendRadioItem(MapFrame_ID_ImageryTheme_DOQ,wxT("USGS Digital Ortho Quadrangle"));
        imageryThemeMenu->AppendRadioItem(MapFrame_ID_ImageryTheme_DRG,wxT("USGS Digital Raster Graphic"));
        imageryThemeMenu->AppendRadioItem(MapFrame_ID_ImageryTheme_Urban,wxT("USGS Urban Area"));
        _viewMenu->Append(MapFrame_ID_ImageryTheme,wxT("Imagery Theme"),imageryThemeMenu);
        
        wxMenu* imageryResMenu=new wxMenu();
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+8,wxT("1/4m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+9,wxT("1/2m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+10,wxT("1m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+11,wxT("2m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+12,wxT("4m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+13,wxT("8m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+14,wxT("16m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+15,wxT("32m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+16,wxT("64m per pixel"));
        imageryResMenu->AppendRadioItem(MapFrame_ID_ImageryRes_0+17,wxT("128m per pixel"));
        _viewMenu->Append(MapFrame_ID_ImageryRes,wxT("Imagery Resolution"),imageryResMenu);
        
        MapUSGSImageLayer::ThemeType imageThemeType=_usgsImageLayer->GetRequestThemeType();
        imageryThemeMenu->Check(MapFrame_ID_ImageryTheme_DOQ,(imageThemeType==MapUSGSImageLayer::ThemeType_DOQ));
        imageryThemeMenu->Check(MapFrame_ID_ImageryTheme_DRG,(imageThemeType==MapUSGSImageLayer::ThemeType_DRG));
        imageryThemeMenu->Check(MapFrame_ID_ImageryTheme_Urban,(imageThemeType==MapUSGSImageLayer::ThemeType_Urban));
        
        int imageScale=_usgsImageLayer->GetRequestScale();
        for (int i=8;i<17;i++) {
                imageryResMenu->Check(MapFrame_ID_ImageryRes_0+i,(imageScale==i));
        }
        
        menuBar->Append(_viewMenu,wxT("&View"));

        _toolsMenu = new wxMenu();
        _toolsMenu->Append(MapFrame_ID_FlipObjectsOrder, wxT("Flip Objects Order"));
        _toolsMenu->Append(MapFrame_ID_VerifyGroundNetwork, wxT("&Verify Ground Network"));
        menuBar->Append(_toolsMenu,wxT("&Tools"));
        
        _projectionsMenu=new wxMenu();
        _projectionsMenu->SetEventHandler(this);
        _projectionsMenu->SetClientData(GetEditor());
        menuBar->Append(_projectionsMenu,wxT("&Projections"));

        
#       if __WXGTK__
        // FIXME: bad hack for working around a bug in wxGTK
        // due to which events do not arrive at the wxMenu elements.
        // There is a FIXME note in the wx-sources since 08/2003 and
        // a request to act on the fixme note was filed on the wxWidgets project
        // as bug 1635404
        // (http://sourceforge.net/tracker/index.php?func=detail&aid=1635404&group_id=9863&atid=109863)
        _projectionsMenu->Detach();
#       endif
        
        SetMenuBar(menuBar);
}

void MapFrame::SetupToolbar() {
        _toolbar=wxDynamicCast(CreateToolBar(wxTB_DOCKABLE),wxMenuToolBar);
        
        _toolbar->AddTool(wxID_NEW,wxGetBitmapFromMemory(icon_new),wxT("New Project..."));
        _toolbar->AddTool(wxID_OPEN,wxGetBitmapFromMemory(icon_open),wxT("Open Existing Project..."));
        _toolbar->AddTool(wxID_SAVE,wxGetBitmapFromMemory(icon_save),wxT("Save Project"));
        _toolbar->AddTool(MapFrame_ID_ImportAirport,wxGetBitmapFromMemory(icon_import),wxT("Import Airport from Database..."));
        _toolbar->AddTool(MapFrame_ID_ExportAirport,wxGetBitmapFromMemory(icon_export),wxT("Export Airport to Database..."));
        _toolbar->AddTool(MapFrame_ID_ImportAINetwork,wxGetBitmapFromMemory(icon_import_ai),wxT("Import AI network..."));
        _toolbar->AddTool(MapFrame_ID_ExportAINetwork,wxGetBitmapFromMemory(icon_export_ai),wxT("Export AI network..."));
        _toolbar->AddSeparator();
        _toolbar->AddTool(wxID_UNDO,wxGetBitmapFromMemory(icon_undo),wxT("Undo one step"));
        _toolbar->AddMenuPulldownTool(MapFrame_ID_UndoMenu, wxT("Undo"));
        _toolbar->AddTool(wxID_REDO,wxGetBitmapFromMemory(icon_redo),wxT("Redo one step"));
        _toolbar->AddMenuPulldownTool(MapFrame_ID_RedoMenu, wxT("Redo"));
        _toolbar->AddSeparator();
        //_toolbar->AddTool(MapFrame_ID_ShowHideLayers,wxGetBitmapFromMemory(icon_showhide),wxT("Show/Hide Layers"));
        //_toolbar->AddTool(MapFrame_ID_LockUnlockLayers,wxGetBitmapFromMemory(icon_lock),wxT("Lock/Unlock layers"));
        _toolbar->AddTool(MapFrame_ID_ZoomIn,wxGetBitmapFromMemory(icon_zoom),wxT("Zoom in"));
        _toolbar->AddTool(MapFrame_ID_ZoomOut,wxGetBitmapFromMemory(icon_unzoom),wxT("Zoom out"));
        _toolbar->AddTool(MapFrame_ID_ZoomFullView,wxGetBitmapFromMemory(icon_zoom_full),wxT("Zoom to full view"));
        _toolbar->AddTool(MapFrame_ID_ZoomToSelection,wxGetBitmapFromMemory(icon_zoom_sel),wxT("Zoom to selection"));
        _toolbar->AddTool(MapFrame_ID_ZoomReset,wxGetBitmapFromMemory(icon_zoom_reset),wxT("Reset zoom to 1:1"));
        _toolbar->AddSeparator();
        _toolbar->AddTool(MapFrame_ID_SelectionMode,wxT("Selection Mode"),wxGetBitmapFromMemory(icon_selection),wxT("Selection Mode"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_RunwayInsertionMode,wxT("Insert Runway"),wxGetBitmapFromMemory(icon_runway),wxT("Insert Runway"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_TaxiwayInsertionMode,wxT("Insert Taxiway"),wxGetBitmapFromMemory(icon_taxiway),wxT("Insert Taxiway"),wxITEM_RADIO);
        //_toolbar->AddTool(MapFrame_ID_SignInsertionMode,wxT("Insert Sign"),wxGetBitmapFromMemory(icon_sign),wxT("Insert Sign"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_BeaconInsertionMode,wxT("Insert Light Beacon"),wxGetBitmapFromMemory(icon_beacon),wxT("Insert Light Beacon"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_StartupLocationInsertionMode,wxT("Insert Startup Location"),wxGetBitmapFromMemory(icon_startuppos),wxT("Insert Startup Location"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_TowerViewpointInsertionMode,wxT("Insert Tower Viewpoint"),wxGetBitmapFromMemory(icon_tower),wxT("Insert Tower Viewpoint"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_WindsockInsertionMode,wxT("Insert Windsock"),wxGetBitmapFromMemory(icon_windsock),wxT("Insert Windsock"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_AINodeInsertionMode,wxT("Insert AI Nodes"),wxGetBitmapFromMemory(icon_ainodes),wxT("Insert AI Nodes"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_AIArcingMode_Unidirectional,wxT("Insert unidirectionally connected AI Nodes"),wxGetBitmapFromMemory(icon_aiarcs_uni),wxT("Insert unidirectionally connected AI Nodes"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_AIArcingMode_Bidirectional,wxT("Insert bidirectionally connected AI Nodes"),wxGetBitmapFromMemory(icon_aiarcs_bi),wxT("Insert bidirectionally connected AI Nodes"),wxITEM_RADIO);
        _toolbar->AddTool(MapFrame_ID_USGSLoadMode,wxT("Load USGS Background Images"),wxGetBitmapFromMemory(icon_usgs),wxT("Load USGS Background Images"),wxITEM_RADIO);
        
        TransactionManager* tmgr=GetAirport()->GetTransactionManager();

        _undoMenuManager=new UndoRedoMenuManager(tmgr,UndoRedoMenuManager::MenuType_Undo);
        _redoMenuManager=new UndoRedoMenuManager(tmgr,UndoRedoMenuManager::MenuType_Redo);
        
        // setup status for undo/redo tools
        OnTransactionStackChange(tmgr);
        
        _toolbar->Realize();
        _toolbar->Show();
}

void MapFrame::SetupDrawingWindow() {
        _drawingWindow=new wxPanel(this, wxID_ANY ,wxDefaultPosition, wxDefaultSize, wxVSCROLL| wxHSCROLL | wxALWAYS_SHOW_SB);
        wxFont drawingFont=wxFont(8,wxFONTFAMILY_TELETYPE,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL);
        _drawingWindow->SetFont(drawingFont);
        _drawingWindow->Show();
}

void MapFrame::SetupMapEditor() {
        MapProjection* proj=UTMProjection::UTM_NAD83.CreateInstance();
        proj->Init(Point3D(0,0,0));

        TransactionManager* tmgr=GetAirport()->GetTransactionManager();
	
	tmgr->AddEditor();
        
        if (!_backgroundImageLayer) {
                _backgroundImageLayer=new MapObjectContainer(
                                                tmgr,
                                                wxT("Background Images"));
        }

        _mapEditor=new MapEditor(_drawingWindow,proj,tmgr);
        
        _mapEditor->GetModeManager().SetStatusField(0);
        
        _mapEditor->PushLayer(_usgsImageLayer)->Lock();
        _mapEditor->PushLayer(_backgroundImageLayer)->Lock();
        _mapEditor->PushLayer(GetAirport()->GetTaxiwayLayer());
        _mapEditor->PushLayer(GetAirport()->GetRunwayLayer())->Lock();
        _mapEditor->PushLayer(GetAirport()->GetGroundNetworkLayer());
        _mapEditor->PushLayer(GetAirport()->GetObjectsLayer());
        
        _mapEditor->PushPass(MapObject::StandardPass);
        _mapEditor->PushPass(PavementMapObject::LightingPass);
        _mapEditor->PushPass(AINetworkVertex::AINetworkPass);
        
        _mapEditor->GetModeManager().InsertOnTop(new CoordinateDisplayMode(GetStatusBar(),1));
        _mapEditor->GetModeManager().InsertOnTop(new PanZoomMode());
        _mapEditor->GetModeManager().InsertOnTop(new PopupMode());
        _mapEditor->GetModeManager().InsertOnTop(new SelectionMode());
        
        tmgr->RegisterStackChangeListener(this);
}

MapFrame::~MapFrame() {
        USGSFetcher::GetInstance()->UnregisterProgressListener(this);
        GetAirport()->GetTransactionManager()->UnregisterStackChangeListener(this);
        GetAirport()->GetTransactionManager()->RemoveEditor();
        delete _undoMenuManager;
        delete _redoMenuManager;
        delete _mapEditor;
}

wxToolBar* MapFrame::OnCreateToolBar(long style, wxWindowID id, const wxString& name) {
        return new wxMenuToolBar(this,id,wxDefaultPosition,wxDefaultSize,wxTB_HORIZONTAL | wxNO_BORDER,name);
}

void MapFrame::OnMenuOpen(wxMenuEvent& evt) {
        wxMenuBar* menuBar=GetMenuBar();
        _viewMenu->Enable(MapFrame_ID_ImageryTheme_DOQ,_usgsImageLayer->HasThemeType(MapUSGSImageLayer::ThemeType_DOQ));
        _viewMenu->Enable(MapFrame_ID_ImageryTheme_DRG,_usgsImageLayer->HasThemeType(MapUSGSImageLayer::ThemeType_DRG));
        _viewMenu->Enable(MapFrame_ID_ImageryTheme_Urban,_usgsImageLayer->HasThemeType(MapUSGSImageLayer::ThemeType_Urban));
        for (int i=8;i<17;i++) {
                _viewMenu->Enable(MapFrame_ID_ImageryRes_0+i,_usgsImageLayer->HasScale(i));
        }
        _viewMenu->Check(MapFrame_ID_ShowProperties,_propertyFrame->IsShown());
        if (evt.GetMenu()==_editMenu) {
                TransactionManager* tmgr=GetEditor()->GetTransactionManager();
                menuBar->Enable(wxID_UNDO,tmgr->CanUndo());
                menuBar->Enable(wxID_REDO,tmgr->CanRedo());
                if (tmgr->CanUndo()) {
                        menuBar->SetLabel(wxID_UNDO,wxT("Undo: ")+
                                    tmgr->GetUndoDescription()+
                                    wxT("\tCTRL+Z"));
                } else {
                        menuBar->SetLabel(wxID_UNDO,wxT("Undo\tCTRL+Z"));
                }
                if (tmgr->CanRedo()) {
                        menuBar->SetLabel(wxID_REDO,wxT("Redo: ")+
                                    tmgr->GetRedoDescription()+
                                    wxT("\tCTRL+R"));
                } else {
                        menuBar->SetLabel(wxID_REDO,wxT("Redo\tCTRL+R"));
                }
                MapSelectionManager& selmgr=GetEditor()->GetSelectionManager();
                menuBar->Enable(wxID_DELETE,(selmgr.GetSelections().GetCount()>0));
                menuBar->Enable(MapFrame_ID_ActionsMenu,(selmgr.GetSelections().GetCount()>0));
                
                DisconnectMenuEvents(_actionsMenu);
                wxMenuItemList list=_actionsMenu->GetMenuItems();
                wxMenuItemList::Node *listNode;
                for (listNode=list.GetFirst();listNode;listNode=listNode->GetNext()) {
                        _actionsMenu->Destroy(listNode->GetData());
                }
                
                size_t separatorPos;
                // Allow the editor to specify its own actions
                _mapEditor->ProvideActionMenu(*_actionsMenu);
                
                if (selmgr.GetSelections().GetCount()>0) {
                // Allow the objects to provide their actions
                        wxClassInfo* commonClass=selmgr.FindCommonSelectionSuperClass();
                        MapSelection* handlerSelection=selmgr.FindFirstSelectionOfType(commonClass);
                        if (handlerSelection!=(MapSelection*)NULL) {
                                separatorPos=_actionsMenu->GetMenuItemCount();
                                handlerSelection->ProvideActionMenuForClass(*_actionsMenu,commonClass);
                                if (separatorPos>0 && separatorPos!=_actionsMenu->GetMenuItemCount())
                                        _actionsMenu->InsertSeparator(separatorPos);
                        }
                }
        } else if (evt.GetMenu()==_viewMenu) {
                MapSelectionManager& selmgr=GetEditor()->GetSelectionManager();
                menuBar->Enable(MapFrame_ID_ZoomToSelection,(selmgr.GetSelections().GetCount()>0));
        } else if (evt.GetMenu()==_projectionsMenu) {
                MapProjectionMenuManager::UpdateMenu(_projectionsMenu);
        }
}

void MapFrame::DisconnectMenuEvents(wxMenu* menu) {
        wxMenuItemList list=menu->GetMenuItems();
        wxMenuItemList::Node *listNode;
        for (listNode=list.GetFirst();listNode;listNode=listNode->GetNext()) {
                while (menu->GetEventHandler()->Disconnect(listNode->GetData()->GetId()))
                        /* intentionally left blank */;
                if (listNode->GetData()->IsSubMenu())
                        DisconnectMenuEvents(listNode->GetData()->GetSubMenu());
        }
}

void MapFrame::OnNewProject(wxCommandEvent& evt) {
        ProjectWizard* wizard = new ProjectWizard(NULL,-1,wxT("TaxiDraw Project Wizard"));
        wizard->RunWizard(wizard->GetTypePage());
        wizard->Destroy();
}

void MapFrame::OnOpenProject(wxCommandEvent& evt) {
        ProjectWizard* wizard = new ProjectWizard(NULL,-1,wxT("TaxiDraw Project Wizard"));
        wizard->GetTypePage()->SetOperation(ProjectWizardTypePage::Open_Project);
        wizard->RunWizard(wizard->GetOpenProjectPage());
        wizard->Destroy();
}

void MapFrame::OnSaveProject(wxCommandEvent& evt) {
        DoSaveProject(evt.GetId()==wxID_SAVEAS);
}

bool MapFrame::DoSaveProject(bool saveAs) {
        wxString file=_projectFilename;
        wxFileName filename;
        if (saveAs || file.IsEmpty()) {
                if (file.IsEmpty()) {
                        filename.AssignCwd();
                        wxString lastSavePath;
                        if (wxConfigBase::Get()->Read(wxT("History/Paths/last-save-path"),
                                                        &lastSavePath))
                        {
                                filename.AssignDir(lastSavePath);
                        }
                        filename.SetName(GetAirport()->GetICAOID());
                        filename.SetExt(wxT("tpj"));
                } else {
                        filename.Assign(file);
                }
                file=::wxFileSelector(wxT("Select a project filename"),
                                          filename.GetPath(),
                                          filename.GetFullName(),
                                          wxT("tpj"),
                                          wxT("TaxiDraw project file (*.tpj)|*.tpj"),
                                          wxSAVE|wxOVERWRITE_PROMPT);
                if (file.IsEmpty()) {
                        return false;
                }
        }
        
        filename.Assign(file);
        
        std::ofstream ostream(file.mb_str(*wxConvFileName));
        OldProjectWriter writer(ostream);
        writer.writeProject(GetAirport(),
                            _mapEditor->GetProjection(),
                            _backgroundImageLayer);
        ostream.close();

        // update the list of recently opened files
        wxArrayString recentlyOpened;
        for (int i=0;;i++) {
                wxString regPath;
                regPath.Printf(wxT("History/Path/recent%d"),i);
                if (!wxConfigBase::Get()->HasEntry(regPath))
                        break;
                wxString fullPath=wxConfigBase::Get()->Read(regPath);
                wxConfigBase::Get()->DeleteEntry(regPath);
                if (fullPath.Cmp(file)==0)
                        continue;
                if (recentlyOpened.GetCount()<9)
                        recentlyOpened.Add(fullPath);
        }
        recentlyOpened.Insert(file,0);
        for (size_t i=0;i<recentlyOpened.GetCount();i++) {
                wxString regPath;
                regPath.Printf(wxT("History/Path/recent%d"),i);
                wxString fullPath=recentlyOpened.Item(i);
                wxConfigBase::Get()->Write(regPath,fullPath);
        }
        
        wxConfigBase::Get()->Write(wxT("History/Paths/last-save-path"),filename.GetPath());
        _projectFilename=file;
        
        GetEditor()->GetTransactionManager()->Clear();
        
        return true;
}

void MapFrame::OnImportAirport(wxCommandEvent& evt) {
        ProjectWizard* wizard = new ProjectWizard(NULL,-1,wxT("TaxiDraw Project Wizard"));
        wizard->GetTypePage()->SetOperation(ProjectWizardTypePage::New_Project);
        wizard->RunWizard(wizard->GetNewProjectPage());
        wizard->Destroy();
}

void MapFrame::OnExportAirport(wxCommandEvent& evt) {
        wxString selFile;
        wxString defaultPath;
        
        defaultPath=wxConfigBase::Get()->Read(wxT("History/Paths/last-export-path"),wxFileName::GetCwd());
        
        selFile=::wxFileSelector(wxT("Select a target file"),
                                  defaultPath,
                                  GetAirport()->GetICAOID()+wxT(".dat"),
                                  wxT("dat"),
                                  wxT("X-Plane .dat files|*.dat"),
                                  wxSAVE|wxOVERWRITE_PROMPT,
                                  this);
        
        if (selFile.IsEmpty()) {
                return;
        }
        
        wxFileName fileName=wxFileName(selFile);
        wxConfigBase::Get()->Write(wxT("History/Paths/last-export-path"),fileName.GetPath());
        
        if (!::wxFileExists(selFile)) {
                std::ofstream ostream(selFile.mb_str(*wxConvFileName));
                XPlaneWriter writer(ostream);
                writer.setVersion(810);
                writer.writeHeader();
                writer.writeAirport(GetAirport());
                writer.writeFooter();
                ostream.close();
        } else {
                wxString outFile=selFile+wxT(".new");
                std::ifstream istream(selFile.mb_str(*wxConvFileName));
                std::ofstream ostream(outFile.mb_str(*wxConvFileName));
                
                XPlaneWriter writer(ostream);
                writer.mergeAirport(istream,GetAirport());
                
                istream.close();
                ostream.close();
                
                ::wxRenameFile(outFile,selFile);
        }
}

void MapFrame::LoadAINetwork(wxString path) {
        string cpath=(const char*)path.mb_str(*wxConvFileName);
        
        std::ifstream istream(cpath.c_str());
        
        wxProgressDialog progDialog(wxT("Loading AI network"),
                                    wxT("Loading AI network"),
                                    0, // no progress gauge
                                    this);
        
        AIGroundNetworkLoader aiNetworkLoader(GetAirport());
        
        readXML(istream,aiNetworkLoader,cpath);
        
        istream.close();

        GetEditor()->GetTransactionManager()->Commit(wxT("Importing AI network"));
        
        GetEditor()->DamageAll();
}

void MapFrame::SaveAINetwork(wxString path) {
        string cpath=(const char*)path.mb_str(*wxConvFileName);

        std::ofstream ostream(cpath.c_str());
        
        AIGroundNetworkWriter aiNetworkWriter(GetAirport());
        
        aiNetworkWriter.write(ostream);
        
        ostream.close();

}

void MapFrame::OnImportAINetwork(wxCommandEvent& evt) {
        wxString fgfsDataPath;
        wxString path;
        
        if (wxConfigBase::Get()->Read(wxT("fgfs-scenery-path"),&fgfsDataPath)) {
                /* Try to find a definition in the FlightGear data directory */
                wxFileName aiNetworkFile;
                aiNetworkFile.AssignDir(fgfsDataPath);
                //aiNetworkFile.AppendDir(wxT("AI"));
                aiNetworkFile.AppendDir(wxT("Airports"));
                wxString ICAO = GetAirport()->GetICAOID();
                //aiNetworkFile.AppendDir(GetAirport()->GetICAOID());
                aiNetworkFile.AppendDir(ICAO[0]);
                aiNetworkFile.AppendDir(ICAO[1]);
                aiNetworkFile.AppendDir(ICAO[2]);
                aiNetworkFile.SetFullName(ICAO + wxT(".groundnet.xml"));
                if (aiNetworkFile.FileExists()) {
                        int ret;
                        
                        ret=::wxMessageBox(wxT("There is a definition of the AI network for this airport ")
                                           wxT("available in the FlightGear scenery directory. ")
                                           wxT("Do you want to load this definition?"),
                                           wxT("Load definition from FlightGear directory?"),
                                           wxYES_NO,
                                           this);
                        if (ret==wxYES) {
                                LoadAINetwork(aiNetworkFile.GetFullPath());
                                return;
                        }
                }
        }
        
        path=wxConfigBase::Get()->Read(wxT("History/Paths/last-ai-open-path"),wxFileName::GetCwd());
        
        path=::wxFileSelector(wxT("Select an AI network file"),
                              path,
                              wxEmptyString,
                              wxT("xml"),
                              wxT("TaxiDraw project files (*.xml)|*.xml"),
                              0,this);
        if (path.empty())
                return;
        
        wxFileName fileName=wxFileName(path);
        wxConfigBase::Get()->Write(wxT("History/Paths/last-ai-open-path"),fileName.GetPath());
        
        LoadAINetwork(path);
}

void MapFrame::OnExportAINetwork(wxCommandEvent& evt) {
        wxString fgfsDataPath;
        wxString path;
        
        if (wxConfigBase::Get()->Read(wxT("fgfs-scenery-path"),&fgfsDataPath) &&
                !fgfsDataPath.IsEmpty()) {
                /* Try to find a definition in the FlightGear data directory */
                int ret;
                
                ret=::wxMessageBox(wxT("Do you want to write the ground network automatically ")
                                   wxT("to the FlightGear scenery directory?"),
                                   wxT("Write definition to FlightGear directory?"),
                                   wxYES_NO,
                                   this);
                if (ret==wxYES) {
                        wxFileName aiNetworkFile;
                        aiNetworkFile.AssignDir(fgfsDataPath);
                        //aiNetworkFile.AppendDir(wxT("AI"));
                        aiNetworkFile.AppendDir(wxT("Airports"));
                        wxString ICAO = GetAirport()->GetICAOID();
                        //aiNetworkFile.AppendDir(GetAirport()->GetICAOID());
                        aiNetworkFile.AppendDir(ICAO[0]);
                        if (!aiNetworkFile.DirExists()) {
                                aiNetworkFile.Mkdir();
                        }
                        aiNetworkFile.AppendDir(ICAO[1]);
                        if (!aiNetworkFile.DirExists()) {
                                aiNetworkFile.Mkdir();
                        }
                        aiNetworkFile.AppendDir(ICAO[2]);
                        if (!aiNetworkFile.DirExists()) {
                                aiNetworkFile.Mkdir();
                        }
                        aiNetworkFile.SetFullName(ICAO + wxT(".groundnet.xml"));
                        SaveAINetwork(aiNetworkFile.GetFullPath());
                        return;
                }
        }
        
        path=wxConfigBase::Get()->Read(wxT("History/Paths/last-ai-open-path"),wxFileName::GetCwd());
        
        path=::wxFileSelector(wxT("Select an AI network file"),
                              path,
                              wxEmptyString,
                              wxT("xml"),
                              wxT("TaxiDraw project files (*.xml)|*.xml"),
                              wxSAVE|wxOVERWRITE_PROMPT,this);
        if (path.empty())
                return;
        
        wxFileName fileName=wxFileName(path);
        wxConfigBase::Get()->Write(wxT("History/Paths/last-ai-open-path"),fileName.GetPath());
        
        SaveAINetwork(path);
}

void MapFrame::OnClose(wxCommandEvent& evt) {
        Close();
}

void MapFrame::OnExit(wxCommandEvent& evt) {
        // FIXME: we should actually close all toplevel frames here
        ::wxExit();
}

void MapFrame::OnUndo(wxCommandEvent& evt) {
        TransactionManager* tmgr=GetEditor()->GetTransactionManager();
        wxASSERT(tmgr->CanUndo());
        tmgr->Undo();
}

void MapFrame::OnRedo(wxCommandEvent& evt) {
        TransactionManager* tmgr=GetEditor()->GetTransactionManager();
        wxASSERT(tmgr->CanRedo());
        tmgr->Redo();
}

void MapFrame::OnDelete(wxCommandEvent& evt) {
        MapSelectionManager &mgr=GetEditor()->GetSelectionManager();
        MapSelectionList selections=mgr.GetSelections();
        MapSelectionList::Node *node;
        for (node=selections.GetFirst();node;node=node->GetNext()) {
                MapObject* obj=node->GetData()->GetContent();
                obj->GetContainer()->DetachObject(obj);
                mgr.RemoveSelection(node->GetData());
        }
        
        TransactionManager* tmgr=GetEditor()->GetTransactionManager();
        tmgr->Commit(wxT("Deleting objects"));
}

void MapFrame::OnDeselectAll(wxCommandEvent& evt) {
        MapSelectionManager &mgr=GetEditor()->GetSelectionManager();
        mgr.DeselectAll();
}

void MapFrame::OnPreferences(wxCommandEvent& evt) {
        PreferenceDialog* prefs=new PreferenceDialog(this,wxID_ANY,
                                                        wxT("Preferences"));
        prefs->ShowModal();
}

void MapFrame::OnNewView(wxCommandEvent& evt) {
        MapFrame* newView;
        newView=new MapFrame(GetAirport(),NULL);
        
        MapEditor *editor=newView->GetEditor();
        
        TDRectangle bounds;
        bounds=newView->GetEditor()->GetGeographicBounds();
        MapProjection* proj=editor->GetProjection();
        Point3D center=bounds.GetCenter();
        proj->Init(center);
        
        bounds=editor->GetLocalBounds();
        editor->ZoomTo(bounds);

        newView->Show();
        MapViewManager::GetInstance()->ActivateView(newView);
}

void MapFrame::OnShowProperties(wxCommandEvent& evt) {
        _propertyFrame->Show(evt.IsChecked());
}

void MapFrame::OnToggleGrid(wxCommandEvent& evt) {
        if (evt.IsChecked()) {
                GetEditor()->GetGrid()->Enable();
        } else {
                GetEditor()->GetGrid()->Disable();
        }
        GetEditor()->DamageAll();
}

void MapFrame::OnZoomIn(wxCommandEvent& evt) {
        GetEditor()->Zoom(1);
}

void MapFrame::OnZoomOut(wxCommandEvent& evt) {
        GetEditor()->Zoom(-1);
}

void MapFrame::OnZoomReset(wxCommandEvent& evt) {
        GetEditor()->SetZoom(1.0);
}

void MapFrame::OnZoomFullView(wxCommandEvent& evt) {
        TDRectangle targetBounds;
        targetBounds=GetEditor()->GetSelectionManager().GetLocalBounds();
        targetBounds=TDRectangle::Union(targetBounds,
                                      GetEditor()->GetLocalBounds());
        
        if (!targetBounds.IsEmpty()) {
                GetEditor()->ZoomTo(targetBounds);
        }
}

void MapFrame::OnZoomToSelection(wxCommandEvent& evt) {
        TDRectangle targetBounds;
        targetBounds=GetEditor()->GetSelectionManager().GetLocalBounds();
        
        if (!targetBounds.IsEmpty()) {
                GetEditor()->ZoomTo(targetBounds);
        }
}

void MapFrame::OnScroll(wxCommandEvent& evt) {
        switch (evt.GetId()) {
        case MapFrame_ID_ScrollUp:
                GetEditor()->ScrollUp();
                break;
        case MapFrame_ID_ScrollDown:
                GetEditor()->ScrollDown();
                break;
        case MapFrame_ID_ScrollLeft:
                GetEditor()->ScrollLeft();
                break;
        case MapFrame_ID_ScrollRight:
                GetEditor()->ScrollRight();
                break;
        }
}

void MapFrame::OnLoadBackgroundImage(wxCommandEvent& evt) {
        wxString bgImgFile;
        wxString default_path;
        default_path=wxConfigBase::Get()->Read(wxT("History/Paths/last-image-path"),
                                               wxFileName::GetCwd());
        bgImgFile=::wxFileSelector(wxT("Select an image file to load"),
                                   default_path,
                                   wxT(""),
                                   wxT("*.jpg"),
                                   wxT("PNG files (*.png)|*.png|JPG files (*.jpg)|*.jpg|All files (*.*)|*.*"),
                                   wxOPEN|wxFILE_MUST_EXIST,
                                   this);
        if (bgImgFile.IsEmpty())
                return;
        wxFileName bgImgFileName=bgImgFile;
        wxConfigBase::Get()->Write(wxT("History/Paths/last-image-path"),bgImgFileName.GetPath());
        
        MapCalibratedBackgroundImage::SmartRef img;
        MapProjection* proj=GetEditor()->GetProjection();
        
        TDRectangle visibleGeoBounds;
        visibleGeoBounds=GetEditor()->GetVisibleGeographicBounds();
        TDRectangle visibleLocalBounds;
        visibleLocalBounds=TDRectangle(proj->ConvertToLocal(visibleGeoBounds.GetTopLeft()),
                                       proj->ConvertToLocal(visibleGeoBounds.GetBottomRight()));
        
        wxProgressDialog progDialog(wxT("Loading background image"),
                                    wxT("Loading background image"),
                                    0, // no progress gauge
                                    this);
        
        img=new MapCalibratedBackgroundImage(GetAirport()->GetTransactionManager(),
                                             visibleGeoBounds.GetTopLeft(),
                                             1.0,
                                             0.0);
        img->SetFilename(bgImgFile);
        const wxImage& imageObj=img->LoadImage();
        
        // precalibrate the image to fit in the currently visible area
        double xscale,yscale,scale;
        xscale=visibleLocalBounds.GetWidth()/imageObj.GetWidth();
        yscale=visibleLocalBounds.GetHeight()/imageObj.GetHeight();
        if (xscale>yscale)
                scale=yscale;
        else
                scale=xscale;

        Point3D localNorthWestPos;
        localNorthWestPos=visibleLocalBounds.GetCenter()+
                Point3D(-imageObj.GetWidth()*scale/2.0,imageObj.GetHeight()*scale/2.0,0.0);

        img->SetNorthWestPos(proj->ConvertFromLocal(localNorthWestPos));
        img->SetScale(scale);
        
        _backgroundImageLayer->AttachObject(img);
        
        // make sure that all preparations are done
        img->GetImage();
        img->Damage();
        
        
        TransactionManager* tmgr=GetAirport()->GetTransactionManager();
        tmgr->Commit(wxT("Loading background image"));
}

void MapFrame::OnVerifyGroundNetwork(wxCommandEvent& evt) {
        AINetworkVerifier networkVerifier(GetAirport(), GetEditor());
        networkVerifier.VerifyAINetwork();
}

void MapFrame::OnMode(wxCommandEvent& evt) {
        if (!evt.IsChecked())
                return;
        
        ActivateMode(evt.GetId());
}

void MapFrame::ActivateMode(int id) {
        GetEditor()->GetModeManager().RemoveAll();
        
        GetToolBar()->ToggleTool(id,true);
        GetMenuBar()->Check(id,true);
        
        if (id==MapFrame_ID_SelectionMode) {
                /* We don't do anything, as the selection mode is always on the stack */
                return;
        }
        
        Mode* theMode;
        
        switch (id) {
        case MapFrame_ID_RunwayInsertionMode:
                theMode=new PavementInsertionMode(GetAirport(),
                                PavementInsertionMode::InsertionMode_InsertRunway);
                break;
        case MapFrame_ID_TaxiwayInsertionMode:
                theMode=new PavementInsertionMode(GetAirport(),
                                PavementInsertionMode::InsertionMode_InsertTaxiway);
                break;
        case MapFrame_ID_BeaconInsertionMode:
                theMode=new ObjectPlacementMode(new BeaconMapObjectPlacer(GetAirport()));
                break;
        case MapFrame_ID_StartupLocationInsertionMode:
                theMode=new ObjectPlacementMode(new StartupLocationMapObjectPlacer(GetAirport()));
                break;
        case MapFrame_ID_TowerViewpointInsertionMode:
                theMode=new ObjectPlacementMode(new TowerMapObjectPlacer(GetAirport()));
                break;
        case MapFrame_ID_WindsockInsertionMode:
                theMode=new ObjectPlacementMode(new WindsockMapObjectPlacer(GetAirport()));
                break;
        case MapFrame_ID_AINodeInsertionMode:
                theMode=new AINetworkNodePlacementMode(GetAirport(),false);
                break;
        case MapFrame_ID_AIArcingMode_Unidirectional:
                theMode=new AINetworkNodePlacementMode(GetAirport(),true,false);
                break;
        case MapFrame_ID_AIArcingMode_Bidirectional:
                theMode=new AINetworkNodePlacementMode(GetAirport(),true,true);
                break;
        case MapFrame_ID_USGSLoadMode: {
                wxFileName imgPath;
                
                // FIXME: prompt the user to enter a path if none was specified yet
                wxString lastPath=wxConfigBase::Get()->Read(wxT("History/Paths/last-download-image-path"),
                        wxFileName::GetCwd());
        
                imgPath.AssignDir(lastPath);
                imgPath.AppendDir(GetAirport()->GetICAOID());
                theMode=new USGSLoadMode(_usgsImageLayer,imgPath);
                break;
        }
        }
        
        theMode->RegisterModeChangeListener(this);
        
        GetEditor()->GetModeManager().InsertOnTop(theMode);
}

void MapFrame::BeforeEnteringMode(Mode* mode) {
        // NO-OP
}

void MapFrame::AfterLeavingMode(Mode* mode) {
        ActivateMode(MapFrame_ID_SelectionMode);
}

void MapFrame::OnImageryThemeType(wxCommandEvent& evt) {
        switch (evt.GetId()) {
        case MapFrame_ID_ImageryTheme_DOQ:
                _usgsImageLayer->SetRequestThemeType(MapUSGSImageLayer::ThemeType_DOQ);
                break;
        case MapFrame_ID_ImageryTheme_DRG:
                _usgsImageLayer->SetRequestThemeType(MapUSGSImageLayer::ThemeType_DRG);
                break;
        case MapFrame_ID_ImageryTheme_Urban:
                _usgsImageLayer->SetRequestThemeType(MapUSGSImageLayer::ThemeType_Urban);
                break;
        }
}

void MapFrame::OnImageryResolution(wxCommandEvent& evt) {
        int scale=evt.GetId()-MapFrame_ID_ImageryRes_0;
        _usgsImageLayer->SetRequestScale(scale);
}

void MapFrame::OnFlipObjectsOrder(wxCommandEvent& evt) {
        GetAirport()->GetRunwayLayer()->FlipObjectsOrder();
        GetAirport()->GetTaxiwayLayer()->FlipObjectsOrder();
        GetAirport()->GetObjectsLayer()->FlipObjectsOrder();
        GetAirport()->GetGroundNetworkLayer()->FlipObjectsOrder();
        
        TransactionManager* tmgr=GetAirport()->GetTransactionManager();
        tmgr->Commit(wxT("Flipping object order"));
}

void MapFrame::OnKeyDown(wxKeyEvent& evt) {
        // ESC triggers exiting of the topmost mode
        if (evt.GetKeyCode()==WXK_ESCAPE && !evt.HasModifiers()) {
                GetEditor()->GetModeManager().RemoveAll();
        } else
                evt.Skip();
}

void MapFrame::OnFillUndoMenu(wxCommandEvent& event) {
        wxMenu* menu=wxStaticCast(event.GetClientData(),wxMenu);
        _undoMenuManager->SetupMenu(menu);
}

void MapFrame::OnFillRedoMenu(wxCommandEvent& event) {
        wxMenu* menu=wxStaticCast(event.GetClientData(),wxMenu);
        _redoMenuManager->SetupMenu(menu);
}

void MapFrame::OnTransactionStackChange(TransactionManager* tmgr) {
        wxASSERT(tmgr==GetAirport()->GetTransactionManager());
        _toolbar->EnableTool(wxID_UNDO,tmgr->CanUndo());
        _toolbar->EnableTool(MapFrame_ID_UndoMenu,tmgr->CanUndo());
        _toolbar->EnableTool(wxID_REDO,tmgr->CanRedo());
        _toolbar->EnableTool(MapFrame_ID_RedoMenu,tmgr->CanRedo());
        SetTitle(wxT("TaxiDraw: ")+
                        GetAirport()->GetICAOID()+wxT(" - ")+GetAirport()->GetName()+
                        (tmgr->CanUndo()?wxT(" *"):wxT("")));
}

void MapFrame::UpdateProgress(unsigned int pending, unsigned int total) {
        _statusBar->SetProgress(total-pending,total);
}

void MapFrame::ActivateView() {
        _propertyFrame->Show(_showPropertyFrameOnActivation);
}

void MapFrame::DeactivateView() {
        _showPropertyFrameOnActivation=_propertyFrame->IsShown();
        _propertyFrame->Show(false);
}

void MapFrame::OnIconize(wxIconizeEvent& evt) {
        if (evt.Iconized()) {
                MapViewManager::GetInstance()->DeactivateView(this);
        } else {
                MapViewManager::GetInstance()->ActivateView(this);
        }
}

void MapFrame::OnActivate(wxActivateEvent& evt) {
        if (evt.GetActive()) {
                MapViewManager::GetInstance()->ActivateView(this);
        }
        evt.Skip();
}

void MapFrame::OnCloseWindow(wxCloseEvent& evt) {
        MapViewManager::GetInstance()->DeactivateView(this);
        if (GetEditor()->GetTransactionManager()->GetUndoSteps()==0) {
                // no unsaved changes
                Destroy();
                return;
        }
        if (GetEditor()->GetTransactionManager()->GetEditorCount()>1) {
                // unsaved changes, but we are not the last editor
                Destroy();
                return;
        }
        // we have unsaved changes and we are the last editor
        int ret=::wxMessageBox(wxT("You have unsaved changes. ")
                               wxT("Do you want to save your changes?"),
                               wxT("Save changes?"),
                               wxYES_NO|(evt.CanVeto()?wxCANCEL:0),
                               this);
        switch (ret) {
        case wxYES:
                if (!DoSaveProject(false) && evt.CanVeto()) {
                        evt.Veto();
                        return;
                }
        case wxNO:
                Destroy();
                return;
        case wxCANCEL:
                evt.Veto();
                return;
        }
}

