#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/config.h>
#include <wx/log.h>

#include <locale.h>
#include <curl/curl.h>

#ifdef _MSC_VER
#  include "TaxiDraw.h"
#endif

#include "TaxiDrawApp.h"

#include "MapFrame.h"
#include "ProjectWizard.h"

#include <AI/AINetworkArc.h>
#include <AI/AINetworkNode.h>
#include <Modes/SelectionMode.h>

#include <Projection/Projections.h>

IMPLEMENT_APP(TDApp)

static void ReadColourConfig(const wxString& path, wxColour *col) {
        wxString str;
        wxConfigBase::Get()->Read(path,&str);
        if (str.IsEmpty())
                return;
        wxString red,green,blue;
        red=str.BeforeFirst(',').Trim();
        str=str.AfterFirst(',');
        green=str.BeforeFirst(',').Trim();
        blue=str.AfterFirst(',').Trim();
        
        unsigned long redv,greenv,bluev;
        if (!red.ToULong(&redv) || !green.ToULong(&greenv) || !blue.ToULong(&bluev))
                return;
        col->Set((unsigned char)redv,(unsigned char)greenv,(unsigned char)bluev);
}

static void WriteColourConfig(const wxString& path, wxColour col) {
        wxString str;
        str.Printf(wxT("%d,%d,%d"),col.Red(),col.Green(),col.Blue());
        wxConfigBase::Get()->Write(path,str);
}

bool TDApp::OnInit()
{
        wxTheApp->SetVendorName(wxT(PACKAGE));
        wxTheApp->SetAppName(wxT(PACKAGE));
        
        wxConfigBase::Set(wxConfigBase::Create());
        
        wxConfigBase::Get()->SetPath(wxT("/"));
        // Don't remove the line below - something *must* be written here to avoid an apparent wxConfig bug on Linux.
        wxConfigBase::Get()->Write(wxT("version"), wxString(VERSION, wxConvUTF8));
        
        // initialise simple preferences
        wxConfigBase::Get()->Read(wxT("Interface/box_selection_includes_intersected"),
                                  &SelectionMode::Include_Intersected);
        ReadColourConfig(wxT("Appearance/map_background_color"),&MapEditor::Map_Background_Colour);
        ReadColourConfig(wxT("Appearance/selection_handle_colour"),&MapSelection::Selection_Handle_Colour);
        ReadColourConfig(wxT("Appearance/selection_marking_colour"),&MapSelection::Selection_Marking_Colour);
        ReadColourConfig(wxT("Appearance/selection_marking_colour"),&MapSelection::Selection_Marking_Colour);
        wxConfigBase::Get()->Read(wxT("Appearance/ai_arc_width"),&AINetworkArc::Arc_Width);
        wxConfigBase::Get()->Read(wxT("Appearance/ai_node_radius"),&AINetworkNode::Radius);

        // Added in response to a bug report that atof was returning int with some locales
        setlocale(LC_NUMERIC, "POSIX");
        
#if wxUSE_LIBJPEG
        wxImage::AddHandler( new wxJPEGHandler );
#endif
#if wxUSE_GIF
        wxImage::AddHandler( new wxGIFHandler );
#endif
        wxImage::AddHandler( new wxPNGHandler );
        
        curl_global_init(CURL_GLOBAL_WIN32);	// The win32 bit flag should be a no-op and hence safe on non-win32 platforms - I *think* !
        
        wxLog* log=new wxLogGui();
        
        wxLog::SetActiveTarget(log);
        
        TDInitAllProjections();
        
        ProjectWizard* wizard = new ProjectWizard(NULL,-1,wxT("TaxiDraw Project Wizard"));
        bool ret=wizard->RunWizard(wizard->GetTypePage());
        wizard->Destroy();
        return ret;
}

int TDApp::OnExit() {
        // write out simple preferences
        wxConfigBase::Get()->Write(wxT("Interface/box_selection_includes_intersected"),
                                   SelectionMode::Include_Intersected);
        WriteColourConfig(wxT("Appearance/map_background_color"),MapEditor::Map_Background_Colour);
        WriteColourConfig(wxT("Appearance/selection_handle_colour"),MapSelection::Selection_Handle_Colour);
        WriteColourConfig(wxT("Appearance/selection_marking_colour"),MapSelection::Selection_Marking_Colour);
        wxConfigBase::Get()->Write(wxT("Appearance/ai_arc_width"),AINetworkArc::Arc_Width);
        wxConfigBase::Get()->Write(wxT("Appearance/ai_node_radius"),AINetworkNode::Radius);
        
        if (wxConfigBase::Get()) {
                wxConfigBase* prefs=wxConfigBase::Set(NULL);
                delete prefs;
        }
	curl_global_cleanup();
	
	return(wxApp::OnExit());
}
