#ifndef MapAngleDraggable_h
#define MapAngleDraggable_h

#include "Map/MapDraggable.h"

/**
 * \brief A MapAngleDraggable is a MapDraggable by which an angle 
 *        between a given center and the cursor position is determined.
 *
 * Angles are given in degrees.
 */
class MapAngleDraggable: public MapDraggable {
public:
        virtual ~MapAngleDraggable() {}
        
        virtual void StartDrag(Point3D currentPosition) {
                _lastAngle=DetermineRelativeAngle(currentPosition);
        }
        
        virtual void Drag(Point3D currentPosition) {
                float angle=DetermineRelativeAngle(currentPosition);
                AngleDrag(angle-_lastAngle);
                _lastAngle=angle;
        }
        
        void SetCenter(Point3D center) { _center=center; }
        
        virtual void AngleDrag(float delta)=0;
protected:
        float DetermineRelativeAngle(Point3D p);
        
        float _lastAngle;
        Point3D _center;
};

#endif

