#ifndef MapLayerInstance_h
#define MapLayerInstance_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "MapLayer.h"

/**
 * \brief A MapLayerInstance represents an instance of a MapLayer in a MapEditor.
 *
 * It contains information about the state of a layer in a specific editor.
 * A layer can be enabled or disabled and locked or unlocked. When it is disabled,
 * it is not painted. When it is locked, no objects in the layer can be selected.
 */
class MapLayerInstance {
public:
        MapLayerInstance(MapLayer::SmartRef layer):
                _layer(layer),
                _enabled(true),
                _locked(false)
        {
        }
        
        virtual ~MapLayerInstance() {
        }
        
        const wxString GetName() const { return _layer->GetName(); }
        
        MapLayer::SmartRef GetLayer() { return _layer; }
        
        const bool IsEnabled() const {
                return _enabled;
        }
        
        void Enable() { _enabled=true; }
        void Disable() { _enabled=false; }
        
        const bool IsLocked() const {
                return _locked;
        }
        
        void Lock() { _locked=true; }
        void Unlock() { _locked=false; }
        
        bool operator==(const MapLayer* l) const {
                return l==_layer;
        }
public:
        MapLayer::SmartRef _layer;
        bool _enabled,_locked;
};

#endif
