// ATCutils.hxx - Utility functions for the ATC / AI subsytem
//
// Written by David Luff, started March 2002.
//
// Copyright (C) 2002  David C Luff - david.luff@nottingham.ac.uk
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//#include <Airports/simple.hxx>
//#include <Airports/runways.hxx>

#include <math.h>
#include "point3d.h"
#include <string>
//SG_USING_STD(string);

using namespace std;

// These are defined here because I had a problem with SG_DEGREES_TO_RADIANS
#define DCL_PI  3.1415926535f
#define DCL_DEGREES_TO_RADIANS  (DCL_PI/180.0)
#define DCL_RADIANS_TO_DEGREES  (180.0/DCL_PI)


/*******************************
*
*  Positional functions
*
********************************/

// Given two positions (lat & lon in degrees), get the HORIZONTAL separation (in meters)
double dclGetHorizontalSeparation(Point3D pos1, Point3D pos2);

// Given a point and a line, get the HORIZONTAL shortest distance from the point to a point on the line.
// Expects to be fed orthogonal co-ordinates, NOT lat & lon !
double dclGetPointLineSeparation(double px, double py, double x1, double y1, double x2, double y2);

double dclGetPointSegmentSeparation(Point3D pt, Point3D p1, Point3D p2);

// Given a position (lat/lon/elev), heading, vertical angle, and distance, calculate the new position.
// Assumes that the ground is not hit!!!  Expects heading and angle in degrees, distance in meters.
Point3D dclUpdatePosition(Point3D pos, double heading, double angle, double distance);

// Get a heading from one lat/lon to another (in degrees)
double GetHeadingFromTo(Point3D A, Point3D B);

// Given a heading (in degrees), bound it from 0 -> 360
void dclBoundHeading(double &hdg);

// smallest difference between two angles in degrees
// difference is negative if a1 > a2 and positive if a2 > a1
double GetAngleDiff_deg( const double &a1, const double &a2);


/*******************************
*
*	Airport-related functions
*
********************************/

// The next two functions are straight copies of their fg.... equivalents
// in fg_init.cxx, and are just here temporarily until some rationalisation occurs.

// find basic airport location info from airport database
//bool dclFindAirportID( const string& id, FGAirport *a );

// get airport elevation
//double dclGetAirportElev( const string& id );

/****************
*
*   Runways
*
****************/

// Given a Point3D (lon/lat/elev) and an FGRunway struct, determine if the point lies on the runway
//bool OnRunway(Point3D pt, const FGRunway& rwy);
