#include "ObjectPlacementMode.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(ObjectPlacementMode,Mode)

BEGIN_EVENT_TABLE(ObjectPlacementMode,Mode)
        EVT_LEFT_DOWN(ObjectPlacementMode::OnLeftDown)
END_EVENT_TABLE()

ObjectPlacementMode::ObjectPlacementMode(MapObjectPlacer* placer):
        _placer(placer)
{
}

ObjectPlacementMode::~ObjectPlacementMode() {
        delete _placer;
}

/* Get a string for the status bar. */
wxString ObjectPlacementMode::GetStatus() {
        return wxT("Click to position a new object");
}

void ObjectPlacementMode::OnLeftDown(wxMouseEvent& evt) {
        if (evt.AltDown() || evt.ControlDown() || evt.ShiftDown()) {
                evt.Skip();
                return;
        }
        
        MapEditor* editor=GetModeManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        MapProjection* proj=editor->GetProjection();
       
        Point3D localPos=cs.DeviceToLogical(evt.GetPosition());
        Point3D geoPos=proj->ConvertFromLocal(localPos);
        
        MapSelectionManager& selmgr=editor->GetSelectionManager();
        selmgr.DeselectAll();
        
        MapObject::SmartRef obj=_placer->PlaceObject(geoPos);
        selmgr.AddSelectionFor(obj);
        
        editor->GetTransactionManager()->Commit(wxT("Object placement"));
}
